-----------------------------------------------------------------------------------------------
-- Client Lua Script for ShortChannelName
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "GameLib"
require "Window"
require "Unit"
require "ChatSystemLib"

-------------------------------------------------------------------------------------
-- ShortChannelName Module Definition
-----------------------------------------------------------------------------------------------
local ShortChannelName = {} 

local shortNames = 
{
	[ChatSystemLib.ChatChannel_Command] 		= "[CM] ", 		
	[ChatSystemLib.ChatChannel_System] 			= "[SM] ", 		
	[ChatSystemLib.ChatChannel_Debug] 			= "[D] ", 			
	[ChatSystemLib.ChatChannel_Say] 			= "[S] ", 			
	[ChatSystemLib.ChatChannel_Yell] 			= "[Y] ", 			
	[ChatSystemLib.ChatChannel_Whisper] 		= "[W] ", 		
	[ChatSystemLib.ChatChannel_Party] 			= "[P] ", 			
	[ChatSystemLib.ChatChannel_Emote] 			= "[E] ", 			
	[ChatSystemLib.ChatChannel_AnimatedEmote] 	= "[E] ", 			
	[ChatSystemLib.ChatChannel_Zone] 			= "[Z] ", 			
	[ChatSystemLib.ChatChannel_ZonePvP] 		= "[PvP] ", 			
	[ChatSystemLib.ChatChannel_Trade] 			= "[T] ",			
	[ChatSystemLib.ChatChannel_Guild] 			= "[G] ", 			
	[ChatSystemLib.ChatChannel_GuildOfficer] 	= "[O] ",	
	[ChatSystemLib.ChatChannel_Society] 		= "[C] ",		
	[ChatSystemLib.ChatChannel_Custom] 			= "[CT] ", 		
	[ChatSystemLib.ChatChannel_NPCSay] 			= "[NPC] ", 			
	[ChatSystemLib.ChatChannel_NPCYell] 		= "[NPC] ",		 	
	[ChatSystemLib.ChatChannel_NPCWhisper]		= "[NPC] ", 			
	[ChatSystemLib.ChatChannel_Datachron] 		= "[NPC] ", 			
	[ChatSystemLib.ChatChannel_Combat] 			= "[CL] ", 		
	[ChatSystemLib.ChatChannel_Realm] 			= "[R] ", 		
	[ChatSystemLib.ChatChannel_Loot] 			= "[L] ", 			
	[ChatSystemLib.ChatChannel_PlayerPath] 		= "[PP] ", 		
	[ChatSystemLib.ChatChannel_Instance] 		= "[I] ", 			
	[ChatSystemLib.ChatChannel_WarParty] 		= "[WP] ",		
	[ChatSystemLib.ChatChannel_WarPartyOfficer] = "[OWP] ", 
	[ChatSystemLib.ChatChannel_Advice] 			= "[A] ", 		
	[ChatSystemLib.ChatChannel_AccountWhisper] 	= "[WA] ",
}

-- Duplicated ChatLog variables

local kcrInvalidColor = ApolloColor.new("InvalidChat")
local kcrValidColor = ApolloColor.new("white")

local kstrColorChatRegular 	= "ff7fffb9"
local kstrColorChatShout	= "ffd9eef7"
local kstrColorChatRoleplay = "ff58e3b0"
local kstrBubbleFont 		= "CRB_Dialog"
local kstrDialogFont 		= "CRB_Dialog"
local kstrDialogFontRP 		= "CRB_Dialog_I"

local kstrGMIcon 		= "Icon_Windows_UI_GMIcon"
local knChannelListHeight = 275

local knWindowStayOnScreenOffset = 50

local knSaveVersion = 3

local karEvalColors =
{
	[Item.CodeEnumItemQuality.Inferior] 		= "ItemQuality_Inferior",
	[Item.CodeEnumItemQuality.Average] 			= "ItemQuality_Average",
	[Item.CodeEnumItemQuality.Good] 			= "ItemQuality_Good",
	[Item.CodeEnumItemQuality.Excellent] 		= "ItemQuality_Excellent",
	[Item.CodeEnumItemQuality.Superb] 			= "ItemQuality_Superb",
	[Item.CodeEnumItemQuality.Legendary] 		= "ItemQuality_Legendary",
	[Item.CodeEnumItemQuality.Artifact]		 	= "ItemQuality_Artifact",
}

local karChannelTypeToColor = -- TODO Merge into one table like this
{
	[ChatSystemLib.ChatChannel_Command] 		= { Channel = "ChannelCommand", 		},
	[ChatSystemLib.ChatChannel_System] 			= { Channel = "ChannelSystem", 			},
	[ChatSystemLib.ChatChannel_Debug] 			= { Channel = "ChannelDebug", 			},
	[ChatSystemLib.ChatChannel_Say] 			= { Channel = "ChannelSay", 			},
	[ChatSystemLib.ChatChannel_Yell] 			= { Channel = "ChannelShout", 			},
	[ChatSystemLib.ChatChannel_Whisper] 		= { Channel = "ChannelWhisper", 		},
	[ChatSystemLib.ChatChannel_Party] 			= { Channel = "ChannelParty", 			},
	[ChatSystemLib.ChatChannel_Emote] 			= { Channel = "ChannelEmote", 			},
	[ChatSystemLib.ChatChannel_AnimatedEmote] 	= { Channel = "ChannelEmote", 			},
	[ChatSystemLib.ChatChannel_Zone] 			= { Channel = "ChannelZone", 			},
	[ChatSystemLib.ChatChannel_ZonePvP] 		= { Channel = "ChannelPvP", 			},
	[ChatSystemLib.ChatChannel_Trade] 			= { Channel = "ChannelTrade",			},
	[ChatSystemLib.ChatChannel_Guild] 			= { Channel = "ChannelGuild", 			},
	[ChatSystemLib.ChatChannel_GuildOfficer] 	= { Channel = "ChannelGuildOfficer",	},
	[ChatSystemLib.ChatChannel_Society] 		= { Channel = "ChannelCircle2",			},
	[ChatSystemLib.ChatChannel_Custom] 			= { Channel = "ChannelCustom", 			},
	[ChatSystemLib.ChatChannel_NPCSay] 			= { Channel = "ChannelNPC", 			},
	[ChatSystemLib.ChatChannel_NPCYell] 		= { Channel = "ChannelNPC",		 		},
	[ChatSystemLib.ChatChannel_NPCWhisper]		= { Channel = "ChannelNPC", 			},
	[ChatSystemLib.ChatChannel_Datachron] 		= { Channel = "ChannelNPC", 			},
	[ChatSystemLib.ChatChannel_Combat] 			= { Channel = "ChannelGeneral", 		},
	[ChatSystemLib.ChatChannel_Realm] 			= { Channel = "ChannelSupport", 		},
	[ChatSystemLib.ChatChannel_Loot] 			= { Channel = "ChannelLoot", 			},
	[ChatSystemLib.ChatChannel_PlayerPath] 		= { Channel = "ChannelGeneral", 		},
	[ChatSystemLib.ChatChannel_Instance] 		= { Channel = "ChannelParty", 			},
	[ChatSystemLib.ChatChannel_WarParty] 		= { Channel = "ChannelWarParty",		},
	[ChatSystemLib.ChatChannel_WarPartyOfficer] = { Channel = "ChannelWarPartyOfficer", },
	[ChatSystemLib.ChatChannel_Advice] 			= { Channel = "ChannelAdvice", 			},
	[ChatSystemLib.ChatChannel_AccountWhisper] 	= { Channel = "ChannelAccountWisper", 	},
}

local ktDefaultChannels =
{
	[ChatSystemLib.ChatChannel_Command] 		= true,
	[ChatSystemLib.ChatChannel_Debug] 			= true,
	[ChatSystemLib.ChatChannel_Say] 			= true,
	[ChatSystemLib.ChatChannel_Yell] 			= true,
	[ChatSystemLib.ChatChannel_Whisper] 		= true,
	[ChatSystemLib.ChatChannel_Party] 			= true,
	[ChatSystemLib.ChatChannel_Emote] 			= true,
	[ChatSystemLib.ChatChannel_AnimatedEmote] 	= true,
	[ChatSystemLib.ChatChannel_Zone]			= true,
	[ChatSystemLib.ChatChannel_ZonePvP] 		= true,
	[ChatSystemLib.ChatChannel_Trade] 			= true,
	[ChatSystemLib.ChatChannel_Guild] 			= true,
	[ChatSystemLib.ChatChannel_GuildOfficer] 	= true,
	[ChatSystemLib.ChatChannel_WarParty] 		= true,
	[ChatSystemLib.ChatChannel_WarPartyOfficer] = true,
	[ChatSystemLib.ChatChannel_Society] 		= true,
	[ChatSystemLib.ChatChannel_Custom] 			= true,
	[ChatSystemLib.ChatChannel_NPCSay] 			= true,
	[ChatSystemLib.ChatChannel_NPCYell] 		= true,
	[ChatSystemLib.ChatChannel_NPCWhisper] 		= true,
	[ChatSystemLib.ChatChannel_Datachron] 		= true,
	[ChatSystemLib.ChatChannel_Realm] 			= true,
	[ChatSystemLib.ChatChannel_Loot] 			= true,
	[ChatSystemLib.ChatChannel_System] 			= true,
	[ChatSystemLib.ChatChannel_PlayerPath] 		= true,
	[ChatSystemLib.ChatChannel_Instance] 		= true,
	[ChatSystemLib.ChatChannel_Advice] 			= true,
	[ChatSystemLib.ChatChannel_AccountWhisper]	= true,
}

local ktChatResultOutputStrings =
{
	[ChatSystemLib.ChatChannelResult_DoesntExist] 			= Apollo.GetString("CRB_Channel_does_not_exist"),
	[ChatSystemLib.ChatChannelResult_BadPassword] 			= Apollo.GetString("CRB_Channel_password_incorrect"),
	[ChatSystemLib.ChatChannelResult_NoPermissions] 		= Apollo.GetString("CRB_Channel_no_permissions"),
	[ChatSystemLib.ChatChannelResult_NoSpeaking] 			= Apollo.GetString("CRB_Channel_no_speaking"),
	[ChatSystemLib.ChatChannelResult_Muted] 				= Apollo.GetString("CRB_Channel_muted"),
	[ChatSystemLib.ChatChannelResult_Throttled] 			= Apollo.GetString("CRB_Channel_throttled"),
	[ChatSystemLib.ChatChannelResult_NotInGroup] 			= Apollo.GetString("CRB_Not_in_group"),
	[ChatSystemLib.ChatChannelResult_NotInGuild] 			= Apollo.GetString("CRB_Channel_not_in_guild"),
	[ChatSystemLib.ChatChannelResult_NotInSociety] 			= Apollo.GetString("CRB_Channel_not_in_society"),
	[ChatSystemLib.ChatChannelResult_NotGuildOfficer] 		= Apollo.GetString("CRB_Channel_not_guild_officer"),
	[ChatSystemLib.ChatChannelResult_AlreadyMember] 		= Apollo.GetString("ChatLog_AlreadyInChannel"),
	[ChatSystemLib.ChatChannelResult_BadName] 				= Apollo.GetString("ChatLog_InvalidChannel"),
	[ChatSystemLib.ChatChannelResult_NotMember] 			= Apollo.GetString("ChatLog_TargetNotInChannel"),
	[ChatSystemLib.ChatChannelResult_NotInWarParty] 		= Apollo.GetString("ChatLog_NotInWarparty"),
	[ChatSystemLib.ChatChannelResult_NotWarPartyOfficer] 	= Apollo.GetString("ChatLog_NotWarpartyOfficer"),
	[ChatSystemLib.ChatChannelResult_InvalidMessageText] 	= Apollo.GetString("ChatLog_InvalidMessage"),
	[ChatSystemLib.ChatChannelResult_InvalidPasswordText] 	= Apollo.GetString("ChatLog_UseDifferentPassword"),
	[ChatSystemLib.ChatChannelResult_TruncatedText]			= Apollo.GetString("ChatLog_MessageTruncated"),
	[ChatSystemLib.ChatChannelResult_InvalidCharacterName]	= Apollo.GetString("ChatLog_InvalidCharacterName"),
	[ChatSystemLib.ChatChannelResult_GMMuted]				= Apollo.GetString("ChatLog_MutedByGm"),
	[ChatSystemLib.ChatChannelResult_MissingEntitlement]	= Apollo.GetString("ChatLog_MissingEntitlement"),
}

local ktChatActionOutputStrings =
{
	[ChatSystemLib.ChatChannelAction_PassOwner] 		= Apollo.GetString("ChatLog_PassedOwnership"),
	[ChatSystemLib.ChatChannelAction_AddModerator] 		= Apollo.GetString("ChatLog_MadeModerator"),
	[ChatSystemLib.ChatChannelAction_RemoveModerator] 	= Apollo.GetString("ChatLog_MadeMember"),
	[ChatSystemLib.ChatChannelAction_Muted] 			= Apollo.GetString("ChatLog_PlayerMuted"),
	[ChatSystemLib.ChatChannelAction_Unmuted] 			= Apollo.GetString("ChatLog_PlayerUnmuted"),
	[ChatSystemLib.ChatChannelAction_Kicked] 			= Apollo.GetString("ChatLog_PlayerKicked"),
	[ChatSystemLib.ChatChannelAction_AddPassword] 		= Apollo.GetString("ChatLog_PasswordAdded"),
	[ChatSystemLib.ChatChannelAction_RemovePassword] 	= Apollo.GetString("ChatLog_PasswordRemoved")
}

local ktChatJoinOutputStrings =
{
	[ChatSystemLib.ChatChannelResult_BadPassword] 			= Apollo.GetString("CRB_Channel_password_incorrect"),
	[ChatSystemLib.ChatChannelResult_AlreadyMember] 		= Apollo.GetString("ChatLog_AlreadyMember"),
	[ChatSystemLib.ChatChannelResult_BadName]				= Apollo.GetString("ChatLog_BadName"),
	[ChatSystemLib.ChatChannelResult_InvalidPasswordText] 	= Apollo.GetString("ChatLog_InvalidPasswordText"),
	[ChatSystemLib.ChatChannelResult_NoPermissions] 		= Apollo.GetString("CRB_Channel_no_permissions"),
	[ChatSystemLib.ChatChannelResult_TooManyCustomChannels]	= Apollo.GetString("ChatLog_TooManyCustom")
}

local ktDatacubeTypeStrings =
{
	[DatacubeLib.DatacubeType_Datacube]						= Apollo.GetString("ChatLog_Datacube"),
	[DatacubeLib.DatacubeType_Chronicle]					= Apollo.GetString("ChatLog_Chronicle"),
	[DatacubeLib.DatacubeType_Journal]						= Apollo.GetString("ChatLog_Journal")
}

local ktDefaultHolds = {}
ktDefaultHolds[ChatSystemLib.ChatChannel_Whisper] = true

--

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ShortChannelName:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ShortChannelName:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

 
-----------------------------------------------------------------------------------------------
-- ShortChannelName OnLoad
-----------------------------------------------------------------------------------------------
function ShortChannelName:OnLoad()
	self:InitializeHooks()
end

function ShortChannelName:InitializeHooks()
	local crb = Apollo.GetAddon("ChatLog")
	if crb == nil then
		crb = Apollo.GetAddon("BetterChatLog")
	end	
	if crb == nil then
		crb = Apollo.GetAddon("ImprovedChatLog")
	end	
	if crb == nil then
		crb = Apollo.GetAddon("ChatAdvanced")
	end			
	if crb ~= nil then
		crb.HelperGenerateChatMessage = self.HelperGenerateChatMessage
	end	
end


function ShortChannelName:HelperGenerateChatMessage(tQueuedMessage)
	if tQueuedMessage.xml then
		return
	end

	local eChannelType = tQueuedMessage.eChannelType
	local tMessage = tQueuedMessage.tMessage

	-- Different handling for combat log
	if eChannelType == ChatSystemLib.ChatChannel_Combat then
		-- no formats in combat, roll it all up into one.
		local strMessage = ""
		for idx, tSegment in ipairs(tMessage.arMessageSegments) do
			strMessage = strMessage .. tSegment.strText
		end
		tQueuedMessage.strMessage = strMessage
		return
	end

	local xml = XmlDoc.new()

	local crText = self.arChatColor[eChannelType] or ApolloColor.new("white")
	local crChannel = ApolloColor.new(karChannelTypeToColor[eChannelType].Channel or "white")
	local crPlayerName = ApolloColor.new("ChatPlayerName")

	local strTime = ""
	if self.bShowTimestamp then
		strTime = self:HelperGetTimeStr()
	end

	local strWhisperName = tMessage.strSender
	if tMessage.strRealmName:len() > 0 and eChannelType ~= ChatSystemLib.ChatChannel_AccountWhisper then
		-- Name/Realm formatting needs to be very specific for cross realm chat to work
		strWhisperName = strWhisperName .. "@" .. tMessage.strRealmName
	end
	local strDisplayName = strWhisperName

	--strWhisperName must only be sender@realm, or friends equivelent name.

	local strPresenceState = ""
	if tMessage.bAutoResponse then
		strPresenceState = '('..Apollo.GetString("AutoResponse_Prefix")..')'
	end

	if tMessage.nPresenceState == FriendshipLib.AccountPresenceState_Away then
		strPresenceState = '<'..Apollo.GetString("Command_Friendship_AwayFromKeyboard")..'>'
	elseif tMessage.nPresenceState == FriendshipLib.AccountPresenceState_Busy then
		strPresenceState = '<'..Apollo.GetString("Command_Friendship_DoNotDisturb")..'>'
	end

	if eChannelType == ChatSystemLib.ChatChannel_Whisper then
		if not tMessage.bSelf then
			self.tLastWhisperer = { strCharacterName = strWhisperName, eChannelType = ChatSystemLib.ChatChannel_Whisper }--record the last incoming whisperer for quick response
		end
		Sound.Play(Sound.PlayUISocialWhisper)
		self:InsertIntoRecent(strWhisperName, false)
	elseif eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then

		local tPreviousWhisperer = self.tLastWhisperer

		self.tLastWhisperer =
		{
			strCharacterName = tMessage.strSender,
			strRealmName = nil,
			strDisplayName = nil,
			eChannelType = ChatSystemLib.ChatChannel_AccountWhisper
		}

		local tAccountFriends = FriendshipLib.GetAccountList()
		for idx, tAccountFriend in pairs(tAccountFriends) do
			if tAccountFriend.arCharacters ~= nil then
				for idx, tCharacter in pairs(tAccountFriend.arCharacters) do
					if tCharacter.strCharacterName == tMessage.strSender and (tMessage.strRealmName:len() == 0 or tCharacter.strRealm == tMessage.strRealmName) then
						if not tMessage.bSelf or (tPreviousWhisperer and tPreviousWhisperer.strCharacterName == tMessage.strSender) then
							self.tLastWhisperer.strDisplayName = tAccountFriend.strCharacterName
							self.tLastWhisperer.strRealmName = tCharacter.strRealm
						end
						strDisplayName = tAccountFriend.strCharacterName
					end
				end
			end
		end
		Sound.Play(Sound.PlayUISocialWhisper)
		self:InsertIntoRecent(strWhisperName, true)
	end

	-- We build strings backwards, right to left
	if eChannelType == ChatSystemLib.ChatChannel_AnimatedEmote then -- emote animated channel gets special formatting
		xml:AddLine(strTime, crChannel, self.strFontOption, "Left")

	elseif eChannelType == ChatSystemLib.ChatChannel_Emote then -- emote channel gets special formatting
		xml:AddLine(strTime, crChannel, self.strFontOption, "Left")
		if strDisplayName:len() > 0 then
			if tMessage.bGM then
				xml:AppendImage(kstrGMIcon, 16, 16)
			end
			local strCross = tMessage.bCrossFaction and "true" or "false"--has to be a string or a number due to code restriction
			xml:AppendText(strDisplayName, crPlayerName, self.strFontOption, {strCharacterName = strWhisperName, nReportId = tMessage.nReportId, strCrossFaction = strCross}, "Source")
		end
		xml:AppendText(" ")
	else
		local strChannel
		if eChannelType == ChatSystemLib.ChatChannel_Society then
			strChannel = string.upper("["..tQueuedMessage.strChannelCommand.."] ") -- Workaround for circles
		elseif eChannelType == ChatSystemLib.ChatChannel_Custom then
			strChannel = string.upper("["..tQueuedMessage.strChannelCommand.."] ") -- Workaround for custom channels
		else
			strChannel = shortNames[eChannelType] -- List of abbreviations applied here
		end

		if self.bShowChannel ~= true then
			strChannel = ""
		end

		xml:AddLine(strTime .. strChannel, crChannel, self.strFontOption, "Left")
		if strDisplayName:len() > 0 then

			local strWhisperNamePrefix = ""
			if eChannelType == ChatSystemLib.ChatChannel_Whisper or eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
				if tMessage.bSelf then
					strWhisperNamePrefix = Apollo.GetString("ChatLog_To")
				else
					strWhisperNamePrefix = Apollo.GetString("ChatLog_From")
				end
			end

			xml:AppendText( strWhisperNamePrefix, crText, self.strFontOption)

			if tMessage.bGM then
				xml:AppendImage(kstrGMIcon, 16, 16)
			end

			local strCross = tMessage.bCrossFaction and "true" or "false"--has to be a string or a number due to code restriction
			xml:AppendText( strDisplayName, crPlayerName, self.strFontOption, {strCharacterName = strWhisperName, nReportId = tMessage.nReportId , strCrossFaction = strCross}, "Source")
		end
		xml:AppendText( strPresenceState .. ": ", crChannel, self.strFontOption, "Left")
	end

	local xmlBubble = nil
	if tMessage.bShowChatBubble then
		xmlBubble = XmlDoc.new() -- This is the speech bubble form
		xmlBubble:AddLine("", crChannel, self.strFontOption, "Center")
	end

	local bHasVisibleText = false
	for idx, tSegment in ipairs( tMessage.arMessageSegments ) do
		local strText = tSegment.strText
		local bAlien = tSegment.bAlien or tMessage.bCrossFaction
		local bShow = false

		if self.eRoleplayOption == 3 then
			bShow = not tSegment.bRolePlay
		elseif self.eRoleplayOption == 2 then
			bShow = tSegment.bRolePlay
		else
			bShow = true;
		end

		if bShow then
			local crChatText = crText;
			local crBubbleText = kstrColorChatRegular
			local strChatFont = self.strFontOption
			local strBubbleFont = kstrBubbleFont
			local tLink = {}


			if tSegment.uItem ~= nil then -- item link
				-- replace me with correct colors
				strText = String_GetWeaselString(Apollo.GetString("CRB_Brackets"), tSegment.uItem:GetName())
				crChatText = karEvalColors[tSegment.uItem:GetItemQuality()]
				crBubbleText = ApolloColor.new("white")

				tLink.strText = strText
				tLink.uItem = tSegment.uItem

			elseif tSegment.uQuest ~= nil then -- quest link
				-- replace me with correct colors
				strText = String_GetWeaselString(Apollo.GetString("CRB_Brackets"), tSegment.uQuest:GetTitle())
				crChatText = ApolloColor.new("green")
				crBubbleText = ApolloColor.new("green")

				tLink.strText = strText
				tLink.uQuest = tSegment.uQuest

			elseif tSegment.uArchiveArticle ~= nil then -- archive article
				-- replace me with correct colors
				strText = String_GetWeaselString(Apollo.GetString("CRB_Brackets"), tSegment.uArchiveArticle:GetTitle())
				crChatText = ApolloColor.new("ffb7a767")
				crBubbleText = ApolloColor.new("ffb7a767")

				tLink.strText = strText
				tLink.uArchiveArticle = tSegment.uArchiveArticle

			else
				if tSegment.bRolePlay then
					crBubbleText = kstrColorChatRoleplay
					strChatFont = self.strRPFontOption
					strBubbleFont = kstrDialogFontRP
				end

				if bAlien or tSegment.bProfanity then -- Weak filter. Note only profanity is scrambled.
					strChatFont = self.strAlienFontOption
					strBubbleFont = self.strAlienFontOption
				end
			end

			if next(tLink) == nil then
				if eChannelType == ChatSystemLib.ChatChannel_AnimatedEmote then
					local strCross = tMessage.bCrossFaction and "true" or "false"--has to be a string or a number due to code restriction
					xml:AppendText(strText, crChatText, strChatFont, {strCharacterName = strWhisperName, nReportId = tMessage.nReportId, strCrossFaction = strCross}, "Source")
				else
					xml:AppendText(strText, crChatText, strChatFont)
				end
			else
				local strLinkIndex = tostring( self:HelperSaveLink(tLink) )
				-- append text can only save strings as attributes.
				xml:AppendText(strText, crChatText, strChatFont, {strIndex=strLinkIndex} , "Link")
			end

			if xmlBubble then
				xmlBubble:AppendText(strText, crBubbleText, strBubbleFont) -- Format for bubble; regular
			end

			bHasVisibleText = bHasVisibleText or self:HelperCheckForEmptyString(strText)
		end
	end

	tQueuedMessage.bHasVisibleText = bHasVisibleText
	tQueuedMessage.xml = xml
	tQueuedMessage.xmlBubble = xmlBubble
end

-----------------------------------------------------------------------------------------------
-- ShortChannelName Instance
-----------------------------------------------------------------------------------------------
local ShortChannelNameInst = ShortChannelName:new()
ShortChannelNameInst:Init()
